# Generated by Django 4.2.16 on 2025-02-10 08:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ddm_projects', '0004_donationproject_url_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='donationproject',
            name='slug',
            field=models.SlugField(help_text='Identifier that is included in the URL through which participants can access the project (e.g, https://root.url/<b>my-url-identifier</b>). Can only contain letters, hyphens, numbers or underscores.', unique=True, verbose_name='URL Identifier'),
        ),
        migrations.AlterField(
            model_name='donationproject',
            name='url_id',
            field=models.SlugField(max_length=8, unique=True),
        ),
    ]
