# Generated by Django 4.2.16 on 2024-12-04 15:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ddm_datadonation', '0004_alter_donationinstruction_text'),
    ]

    operations = [
        migrations.AlterField(
            model_name='donationblueprint',
            name='csv_delimiter',
            field=models.CharField(blank=True, default='', help_text='This field allows you to specify the character that separates values in the expected CSV file (e.g., , ; or \\t). If left empty, DDM will try to infer the delimiter from the file structure.', max_length=10),
        ),
        migrations.AlterField(
            model_name='donationblueprint',
            name='expected_fields_regex_matching',
            field=models.BooleanField(default=False, help_text='Select if you use regex expressions in the "Expected fields".'),
        ),
    ]
