# Generated by Django 4.2.16 on 2024-11-07 10:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ddm_projects', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='donationproject',
            name='briefing_text',
            field=models.TextField(default='', help_text='This text will be displayed to participants in the first step before beginning the data donation. Here, you should introduce your study, who is responsible for the study, provide a quick summary of what your participants will be asked to do in the next steps, etc.', verbose_name='Briefing Text'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='donationproject',
            name='contact_information',
            field=models.TextField(default='', help_text='Please provide the contact information of the person responsible for the conduction of this study (Name, professional address, e-mail, tel, ...). The contact information will be accessible for participants during the data donation. The field is mandatory.', verbose_name='Contact information'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='donationproject',
            name='data_protection_statement',
            field=models.TextField(default='', help_text='Please provide a data protection statement for your data donation collection. This should include the purpose for which the data is collected, how it will be stored, and who will have access to the data. The field is mandatory.', verbose_name='Data Protection Statement'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='donationproject',
            name='debriefing_text',
            field=models.TextField(default='', help_text='This text will be displayed to participants on the last page of the study (i.e., after the data donation and the questionnaire).', verbose_name='Debriefing Text'),
            preserve_default=False,
        ),
    ]
