.ddm-message {
    border-radius: 5px;
    padding: 5px 10px 5px 10px;
    font-size: 0.9rem;
    width: 350px;
    max-width: 30%;
    margin-bottom: 5px;
    opacity: 0;
    position: relative;
    top: 50px;
    z-index: 100;
}

.ddm-message-container {
    position: absolute;
    top: 0;
    right: 0;
}

.ddm-message-container .success {
    background: #c9eae8;
    border: 0;
    color: #009c94;
}

.ddm-message-close {
    color: #009c94 !important;
}

.ddm-message-close:hover {
    color: #005752 !important;
}
