.ddm-info-collapsible {
    border-left: 2px solid black;
    border-right: 2px solid black;
    margin: 15px 10PX 25px 10px;
}

.ddm-info-collapsible-header {
    background: #f2f2f2;
}

.ddm-info-collapsible-button {
    padding: 5px 15px;
    background: none;
    border: none;
    width: 100%;
    text-align: left;
}

.ddm-info-collapsible-button:after {
    content: "\F229";
    font-family: "bootstrap-icons";
    display: inline-block;
    float: right;
    transition-duration: .35s;
}

.ddm-info-collapsible-button:not(.collapsed):after {
    content: "\F229";
    font-family: "bootstrap-icons";
    display: inline-block;
    transform: rotate(-180deg);
    transition-duration: .35s;
    float: right;
}

.ddm-info-collapsible-body {
    padding: 15px 21px;
    border-bottom: 1px solid lightgrey;
}

.ddm-info-collapsible-body ol {
    line-height: 1.5rem;
}

.ddm-info-collapsible-body li {
    padding-left: 5px;
}
