# Generated by Django 3.2.13 on 2024-10-25 08:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ddm', '0048_alter_eventlogentry_description'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations = [
                migrations.DeleteModel(
                    name = 'ProjectAccessToken',
                ),
            ],
            database_operations = [
                migrations.AlterModelTable(
                    name = 'ProjectAccessToken',
                    table = 'ddm_auth_projectaccesstoken',
                ),
            ],
        )
    ]
