# Generated by Django 4.2.16 on 2024-11-07 10:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('ddm_questionnaire', '0005_auto_20241026_1048'),
    ]

    operations = [
        migrations.AlterField(
            model_name='questionbase',
            name='polymorphic_ctype',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_%(app_label)s.%(class)s_set+', to='contenttypes.contenttype'),
        ),
        migrations.AlterField(
            model_name='questionbase',
            name='text',
            field=models.TextField(blank=True, default='', help_text='If a question is linked to a File Blueprint, data points from the donated data associated with the linked donation blueprint can be included in the question text. This data can be included as "{{ data }}" in the question text. It is possible to subset the data object (e.g., to include the last datapoint you can use {{ data.0 }} or include advanced rendering options included in the Django templating engine. For a more comprehensive overview and examples see the documentation. Additionally, information directly related to the participant can be included in the question text. This information can be referenced as "{{ participant }}".'),
            preserve_default=False,
        ),
    ]
