# Generated by Django 3.2.13 on 2024-10-25 15:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ddm', '0053_auto_20241025_1451'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.RemoveField(
                    model_name='participant',
                    name='project',
                ),
            ],
            database_operations=[]
        ),
    ]
