/* Basic ddm form styling. */
.ddm-form {
    font-size: 0.9rem;
}

.ddm-form label {
    display: block;
    font-size: 0.8rem;
    font-weight: bold;
}

.ddm-form p {
    margin-bottom: 0;
    margin-top: 1rem;
}

.ddm-form .helptext,
.helptext {
    color: grey;
    font-size: 0.8rem;
}

.ddm-form .ddm-btn {
    margin-top: 1rem;
}

.ddm-form h5, h6 {
    margin-top: 2rem;
}

.ddm-form input[type=text],
.ddm-form input[type=number],
.ddm-form input[type=password],
.ddm-form input[type=email],
.ddm-form textarea {
    border-radius: 3px;
    border: 1px solid #ddd;
    padding: 5px;
    width: 50%;
    display: block;
}

.ddm-form input:focus,
.ddm-form textarea:focus,
.ddm-form select:focus {
    outline: 1px solid #009c94;
    border-color: white;
}

.ddm-form select {
    background: white;
    border: 1px solid #ddd;
    border-radius: 3px;
    height: 30px;
    display: block;
}

/* Custom checkbox input: */
.ddm-form input[type=checkbox] {
    display: none;
}

.checkmark {
    position: relative;
    height: 20px;
    width: 20px;
    background-color: #fff;
    border: 1px solid #ddd;
    border-radius: 3px;
}

.checkmark:hover {
    cursor: pointer;
}

.ddm-form input[type=checkbox]:checked + .checkmark {
    background-color: #009c94;
}

.checkmark:after {
    content: "";
    position: absolute;
    display: none;
}

.ddm-form input:checked + .checkmark:after {
    display: block;
}

.ddm-form .checkmark:after {
    left: 7px;
    top: 3px;
    width: 6px;
    height: 10px;
    border: solid white;
    border-width: 0 3px 3px 0;
    -webkit-transform: rotate(35deg);
    -ms-transform: rotate(35deg);
    transform: rotate(35deg);
}

.ddm-choice-label {
    font-weight: normal !important;
    font-size: 1rem !important;
    margin-left: 5px;
    cursor: pointer;
}

.ddm-form-file-field label {
    display: inline-block;
}

/* Auth form - used on login page. */
.auth-form {
    font-size: 1rem !important;
}

.auth-form input {
    width: 75% !important;
    display: inline !important;
}

.auth-form input {
    display: block !important;
    margin-left: auto;
    margin-right: auto;
}

.auth-form ul {
    width: 75%;
    margin-left: auto;
    margin-right: auto;
    text-align: left;
    font-size: 0.8rem;
    color: gray;
}

.django-ckeditor-widget {
    display: block !important;
}
