# Generated by Django 3.2.13 on 2024-10-25 09:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ddm', '0049_delete_projectaccesstoken'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.RemoveField(
                    model_name='exceptionlogentry',
                    name='blueprint',
                ),
                migrations.RemoveField(
                    model_name='exceptionlogentry',
                    name='participant',
                ),
                migrations.RemoveField(
                    model_name='exceptionlogentry',
                    name='project',
                ),
                migrations.DeleteModel(
                    name='EventLogEntry',
                ),
                migrations.DeleteModel(
                    name='ExceptionLogEntry',
                ),
            ],
            database_operations=[
                migrations.AlterModelTable(
                    name='EventLogEntry',
                    table='ddm_logging_eventlogentry'
                ),
                migrations.AlterModelTable(
                    name='ExceptionLogEntry',
                    table='ddm_logging_exceptionlogentry'
                ),
            ]
        )
    ]
