{% extends "ddm_participation/base.html" %}
{% load static i18n %}

{% block content %}

  <h1>{% translate "Participation not Possible" %}</h1>
  <p>
    {% translate "Unfortunately, you cannot participate in this study at this time. This may be because the project has not started or the study has already concluded." %}
  </p>

{% endblock %}
