{% extends 'ddm_core/page_with_form.html' %}

{% block page_title %}Edit Instruction Page{% endblock %}

{% block main_heading %}Edit Instruction Page for File Uploader "{{ file_uploader.name }}"{% endblock %}

{% block submit_label %}Save Instruction Page{% endblock %}

{% block cancel %}
<p>
  <a href="{% url 'ddm_datadonation:instructions:overview' project_url_id file_uploader.pk %}">&#129040; Back</a>
</p>
{% endblock %}

{% block breadcrumbs %}
<a href="{% url 'ddm_projects:list' %}">Projects</a> /
<a href="{% url 'ddm_projects:detail' project_url_id %}">"{{ file_uploader.project.name|truncatechars:15 }}" Project</a> /
<a href="{% url 'ddm_datadonation:overview' project_url_id %}">Data Donation</a> /
<a href="{% url 'ddm_datadonation:uploaders:edit' project_url_id file_uploader.pk %}">File Uploader "{{ file_uploader.name|truncatechars:15 }}"</a> /
<a href="{% url 'ddm_datadonation:instructions:overview' project_url_id file_uploader.pk %}">Instructions</a> /
Edit Instruction Page
{% endblock %}
