<p>
  General questions will be displayed to all participants after they have concluded the data donation.
</p>
<p>
  Questions associated to a file blueprint will only be shown to those participants who have donated some data to the respective blueprint.
  Otherwise, they will be omitted.<br>
  In questions associated to a file blueprint, the donated data is available as a variable through which specific data points can be extracted from the donated data
  of a person and included in the question body (e.g., extract the last video from a person's watch history and ask questions related to this specific video).
</p>
