# Generated by Django 4.2.16 on 2025-02-15 11:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ddm_questionnaire', '0007_alter_questionitem_label_alt'),
    ]

    operations = [
        migrations.AddField(
            model_name='openquestion',
            name='input_type',
            field=models.CharField(choices=[('text', 'Any text'), ('numbers', 'Numbers only'), ('email', 'Email address')], default='text', help_text='Select the type of input allowed.', max_length=20, verbose_name='Input type'),
        ),
        migrations.AddField(
            model_name='openquestion',
            name='max_input_length',
            field=models.IntegerField(blank=True, default=None, help_text="Participants' input cannot exceed this length. If empty, no input length restriction is enforced.", null=True, verbose_name='Maximum input length'),
        ),
        migrations.AlterField(
            model_name='openquestion',
            name='display',
            field=models.CharField(choices=[('small', 'Small'), ('large', 'Large')], default='small', help_text='"Small" displays a one-line textfield, "Large" a multiline textfield as input.', max_length=20),
        ),
    ]
