{% extends 'ddm_core/page_with_form.html' %}

{% block page_title %}Delete Uploader{% endblock %}

{% block main_heading %}Delete Uploader "{{ object.name }}"{% endblock %}

{% block main_top %}
<p>You are about to delete the following Uploader:</p>

<div>
  <p>
    <label>Name:</label>
    {{ object.name }}
  </p>
</div>

<div class="mt-4">
  <p>The donation instructions associated with this Uploader will also be deleted. The associated file blueprints will not be deleted.</p>
  <div>
    <p>The following instructions are associated with this Uploader:</p>
    <div>
      <p><b>Instructions:</b></p>
      <ul>
        {% for instruction in object.instruction_set.all %}
        <li>{{ instruction }}</li>
        {% empty %}
        No instructions associated to this uploader were found.
        {% endfor %}
      </ul>
    </div>
  </div>
</div>

<form method='post'>
  {% csrf_token %}
  <p><b>Are you sure you want to delete this Uploader along with all associated instructions? This action cannot be reversed.</b></p>
  <input class="ddm-btn ddm-btn-red" type="submit" value="Delete Uploader">
  <p class="mt-3">
    <a href="{% url 'ddm_datadonation:overview' project.url_id %}">&#129040; Cancel</a>
  </p>
</form>
{% endblock %}

{% block main_form %}{% endblock %}


{% block breadcrumbs %}
<a href="{% url 'ddm_projects:list' %}">Projects</a> /
<a href="{% url 'ddm_projects:detail' project.url_id %}">"{{ project.name|truncatechars:15 }}" Project</a> /
<a href="{% url 'ddm_datadonation:overview' project.url_id %}">Data Donation</a> /
Delete Uploader
{% endblock %}
