# Generated by Django 3.2.13 on 2024-10-25 10:09

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ddm', '0050_auto_20241025_1119'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.RemoveField(
                    model_name='multichoicequestion',
                    name='questionbase_ptr',
                ),
                migrations.RemoveField(
                    model_name='openquestion',
                    name='questionbase_ptr',
                ),
                migrations.RemoveField(
                    model_name='questionbase',
                    name='blueprint',
                ),
                migrations.RemoveField(
                    model_name='questionbase',
                    name='polymorphic_ctype',
                ),
                migrations.RemoveField(
                    model_name='questionbase',
                    name='project',
                ),
                migrations.RemoveField(
                    model_name='questionitem',
                    name='question',
                ),
                migrations.RemoveField(
                    model_name='questionnaireresponse',
                    name='participant',
                ),
                migrations.RemoveField(
                    model_name='questionnaireresponse',
                    name='project',
                ),
                migrations.RemoveField(
                    model_name='scalepoint',
                    name='question',
                ),
                migrations.RemoveField(
                    model_name='semanticdifferential',
                    name='questionbase_ptr',
                ),
                migrations.RemoveField(
                    model_name='singlechoicequestion',
                    name='questionbase_ptr',
                ),
                migrations.RemoveField(
                    model_name='transition',
                    name='questionbase_ptr',
                ),
                migrations.DeleteModel(
                    name='MatrixQuestion',
                ),
                migrations.DeleteModel(
                    name='MultiChoiceQuestion',
                ),
                migrations.DeleteModel(
                    name='OpenQuestion',
                ),
                migrations.DeleteModel(
                    name='QuestionBase',
                ),
                migrations.DeleteModel(
                    name='QuestionItem',
                ),
                migrations.DeleteModel(
                    name='QuestionnaireResponse',
                ),
                migrations.DeleteModel(
                    name='ScalePoint',
                ),
                migrations.DeleteModel(
                    name='SemanticDifferential',
                ),
                migrations.DeleteModel(
                    name='SingleChoiceQuestion',
                ),
                migrations.DeleteModel(
                    name='Transition',
                ),
            ],
            database_operations=[
                migrations.AlterModelTable(
                    name='MatrixQuestion',
                    table='ddm_questionnaire_matrixquestion'
                ),
                migrations.AlterModelTable(
                    name='MultiChoiceQuestion',
                    table='ddm_questionnaire_multichoicequestion'
                ),
                migrations.AlterModelTable(
                    name='OpenQuestion',
                    table='ddm_questionnaire_openquestion'
                ),
                migrations.AlterModelTable(
                    name='QuestionBase',
                    table='ddm_questionnaire_questionbase'
                ),
                migrations.AlterModelTable(
                    name='QuestionItem',
                    table='ddm_questionnaire_questionitem'
                ),
                migrations.AlterModelTable(
                    name='QuestionnaireResponse',
                    table='ddm_questionnaire_questionnaireresponse'
                ),
                migrations.AlterModelTable(
                    name='ScalePoint',
                    table='ddm_questionnaire_scalepoint'
                ),
                migrations.AlterModelTable(
                    name='SemanticDifferential',
                    table='ddm_questionnaire_semanticdifferential'
                ),
                migrations.AlterModelTable(
                    name='SingleChoiceQuestion',
                    table='ddm_questionnaire_singlechoicequestion'
                ),
                migrations.AlterModelTable(
                    name='Transition',
                    table='ddm_questionnaire_transition'
                ),
            ]
        )
    ]
