# Generated by Django 4.2.16 on 2024-12-05 13:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ddm_questionnaire', '0006_alter_questionbase_polymorphic_ctype_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='questionitem',
            name='label_alt',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Label Right'),
        ),
    ]
