# Generated by Django 3.2.13 on 2024-10-25 15:49

import ddm.participation.models
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('ddm', '0054_remove_participant_project'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name='Participant',
                    fields=[
                        ('id',
                         models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                        ('external_id', models.CharField(max_length=24, unique=True,
                                                         validators=[django.core.validators.MinLengthValidator(24)])),
                        ('start_time', models.DateTimeField()),
                        ('end_time', models.DateTimeField(null=True)),
                        ('completed', models.BooleanField(default=False)),
                        ('current_step', models.IntegerField(blank=True, null=True)),
                        ('extra_data', models.JSONField(default=ddm.participation.models.get_extra_data_default)),
                        ('project',
                         models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='ddm.donationproject')),
                    ],
                ),
            ],
            # Table already exists. See ddm/migrations/0054, 0055.
            database_operations=[]
        )
    ]
